#!/bin/env python3

def foo1():
    pass

if toto == True:
    def foo2():
        pass
else:
    def foo2():
        pass

class class1:
    pass

def foo3(param):
    pass

def foo4(param1, param2="default"):
    pass

class class2(class1):
    pass

class class3(class1, toto=1):
    pass

class class4():
    def foo1_in_class4():
        pass
